<template>
  <Dialog v-model="dialogVisible" :title="dialogTitle">
    <el-form
      ref="formRef"
      v-loading="formLoading"
      :model="formData"
      :rules="formRules"
      label-width="80px"
    >
      <el-form-item label="名称" prop="name">
        <el-input v-model="formData.name" placeholder="请输入名称" />
      </el-form-item>
      <el-form-item label="数据源" prop="dataSource">
        <el-select v-model="formData.dataSource" clearable placeholder="请选择数据源">
          <el-option
            v-for="item in dataSourceList"
            :key="item.id"
            :label="item.name"
            :value="item.id + ''"
          />
        </el-select>
      </el-form-item>
      <el-form-item label="查询语句" prop="querySql">
        <el-input v-model="formData.querySql" placeholder="请输入内容" type="textarea" maxlength="300"
                  show-word-limit :rows="6" @input="checkSensitiveWords" spellcheck="false"/>
      </el-form-item>
      <el-form-item v-if="showError">
        <p>输入中包含以下敏感词：<span style="color: red">{{sensitiveMessage}}</span></p>
      </el-form-item>
      <el-form-item label="备注" prop="remark">
        <el-input v-model="formData.remark" placeholder="请输入内容" type="textarea" maxlength="100"
                  show-word-limit/>
      </el-form-item>
    </el-form>
    <template #footer>
      <el-button :disabled="disableSubmit" type="primary" @click="submitForm">确 定</el-button>
      <el-button @click="dialogVisible = false">取 消</el-button>
    </template>
  </Dialog>
</template>
<script lang="ts" setup>
import * as DataSetApi from '@/api/data/plan/data'
import * as DataSourceConfigApi from "@/api/infra/dataSourceConfig";

defineOptions({ name: 'PlanDataSetForm' })

const { t } = useI18n() // 国际化
const message = useMessage() // 消息弹窗

const showError = ref(false)
const foundSensitiveWords = ref()
const sensitiveMessage = ref('')
const sensitiveWords = [';', 'master', 'truncate', 'insert', 'select', 'delete', 'update', 'declare', 'alter', 'drop']

const dialogVisible = ref(false) // 弹窗的是否展示
const dialogTitle = ref('') // 弹窗的标题
const formLoading = ref(false) // 表单的加载中：修改时的数据加载；
const disableSubmit = ref(false) // 禁止提交
const formType = ref('') // 表单的类型：create - 新增；update - 修改
const formData = ref({
  id: undefined,
  name: '',
  dataSource: '',
  querySql: '',
  remark: ''
})
const formRules = reactive({
  name: [{ required: true, message: '名称不能为空', trigger: 'blur' }],
  dataSource: [{ required: true, message: '数据源不能为空', trigger: 'blur' }],
  querySql: [{ required: true, message: '查询语句不能为空', trigger: 'change' }]
})
const formRef = ref() // 表单 Ref
const dataSourceList = ref([] as DataSourceConfigApi.DataSourceConfigVO[])

/** 打开弹窗 */
const open = async (type: string, id?: number) => {
  dialogVisible.value = true
  dialogTitle.value = t('action.' + type)
  formType.value = type
  resetForm()

  // 加载数据源列表
  dataSourceList.value = await DataSourceConfigApi.getDataSourceConfigList()

  // 修改时，设置数据
  if (id) {
    formLoading.value = true
    try {
      formData.value = await DataSetApi.getDataSet(id)
    } finally {
      formLoading.value = false
    }
  }
}
defineExpose({ open }) // 提供 open 方法，用于打开弹窗

/**
 * 验证敏感词
 */
const checkSensitiveWords = () => {
  showError.value = false;
  const regex = new RegExp(sensitiveWords.map(word => `${word}`).join('|'), 'gi');
  let matches = formData.value.querySql.match(regex);
  if (matches) {
    showError.value = true;
    foundSensitiveWords.value = Array.from(new Set(matches));
    disableSubmit.value = true
    sensitiveMessage.value = foundSensitiveWords.value.join('、')
  } else {
    foundSensitiveWords.value = undefined
    disableSubmit.value = false
  }
}

/** 提交表单 */
const emit = defineEmits(['success']) // 定义 success 事件，用于操作成功后的回调
const submitForm = async () => {
  // 校验表单
  if (!formRef) return
  const valid = await formRef.value.validate()
  if (!valid) return
  // 提交请求
  formLoading.value = true
  disableSubmit.value = true
  try {
    const data = formData.value as DataSetApi.DataSetVO
    if (formType.value === 'create') {
      await DataSetApi.createDataSet(data)
      message.success(t('common.createSuccess'))
    } else {
      await DataSetApi.updateDataSet(data)
      message.success(t('common.updateSuccess'))
    }
    dialogVisible.value = false
    // 发送操作成功的事件
    emit('success')
  } finally {
    formLoading.value = false
    disableSubmit.value = false
  }
}

/** 重置表单 */
const resetForm = () => {
  formData.value = {
    id: undefined,
    name: '',
    dataSource: '',
    querySql: '',
    remark: ''
  }
  formRef.value?.resetFields()
}
</script>
